<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Helper;

use Magento\Framework\App\ResourceConnection;

/**
 * Class Indexer
 * @package FiloBlu\Refilo\Helper
 */
class Indexer
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * Indexer constructor.
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(ResourceConnection $resourceConnection)
    {
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * @param $table
     * @param $idColumn
     * @return array
     */
    public function getDeletedEntityIds($table, $idColumn)
    {
        return $this->getDeletedEntityIdsByIntervals($this->getDeletedEntityIntervals($table, $idColumn));
    }

    /**
     * @param array $intervals
     * @return array
     */
    public function getDeletedEntityIdsByIntervals(array $intervals)
    {
        $ids = [];

        $end = null;
        foreach ($intervals as $interval) {
            if ($end === null) {
                $end = $interval['end'];
                continue;
            }

            $ids = array_merge($ids, range($end + 1, $interval['start'] - 1));
            $end = $interval['end'];
        }

        return $ids;
    }

    /**
     * @param $table
     * @param $idColumn
     * @return array
     */
    public function getDeletedEntityIntervals($table, $idColumn)
    {
        return $this->resourceConnection->getConnection()
            ->fetchAll("SELECT l.{$idColumn} AS `start`, l.{$idColumn} +2 AS `end` FROM {$table} AS l LEFT OUTER JOIN  {$table} AS r ON l.{$idColumn} + 1 = r.{$idColumn} WHERE r.{$idColumn} IS NULL");
    }
}
